@@ -1,6 +1,6 @@
 /*
 integrit - file integrity verification system
-Copyright (C) 2000 Ed Cashin
+Copyright (C) 2001 Ed Cashin
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -36,6 +36,28 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 /* Define this if your system provides strerror */
 #undef	HAVE_STRERROR
 
+/* define this on a host with little-endian byte order */
+#undef LITTLE_ENDIAN_HOST
+
+/* define this on a host with big-endian byte order */
+#undef BIG_ENDIAN_HOST
+
+/* biggest file size in bytes that cdb-reading code will mmap */
+#undef CDB_MMAP_MAX
+
+/* ---------------- gnupg crypto support ------------ */
+/* define when system provides typedef for byte */
+#undef HAVE_BYTE_TYPEDEF
+/* define when system provides typedef for byte */
+#undef HAVE_USHORT_TYPEDEF
+/* define when system provides typedef for byte */
+#undef HAVE_ULONG_TYPEDEF
+/* define when system provides typedef for byte */
+#undef HAVE_U16_TYPEDEF
+/* define when system provides typedef for byte */
+#undef HAVE_U32_TYPEDEF
+
+
 /* (end acconfig.h) */
 
 @TOP@
