@@ -17,6 +17,7 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 */
+#include	<config.h>
 #include	<stdio.h>
 #include	<string.h>
 #include	<stdlib.h>
@@ -25,14 +26,26 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include	<limits.h>
 #include	<errno.h>
 #include	<sys/stat.h>
-#include	<openssl/sha.h>
+/* support platforms that don't yet conform to C99 */
+#if	HAVE_STDINT_H
+#include	<stdint.h>
+#elif	HAVE_INTTYPES_H
+#include	<inttypes.h>
+#else
+#error No stdint.h or inttypes.h found.
+#endif
+#include	"gnupg/sha1.h"
 #include	"elcerror_p.h"
 #include	"elcerror.h"
-#include	"cdb_seq.h"
 #include	"cdb.h"
+#include	"cdb_seq.h"
 #include	"dbinfo.h"
 #include	"show.h"
+#ifdef		ELC_FIND_LEAKS
+#include	"leakfind.h"
+#endif
 
+#undef	PROGNAME
 #define	PROGNAME "i-viewdb"
 
 typedef struct options {
@@ -68,7 +81,9 @@ void viewdb(options *opts)
 	  DIE("key size too big");
 	ksiz	 *= 2;
 	if (! (key = realloc(key, ksiz)) )
-	  DIE("realloc key");
+	  die(__FUNCTION__,
+	      "Error: realloc key (length %d) with size (%d): %s",
+	      klen, ksiz, strerror(errno));
       }
       if (vlen > sizeof(val))
 	die(__FUNCTION__, "Error: bad entry (too big value) in DB (%s)",
@@ -108,6 +123,9 @@ int main(int argc, char *argv[])
       usage();
       exit(EXIT_FAILURE);
     }
-    
+
+#ifdef		ELC_FIND_LEAKS
+    GC_gcollect();		/* find the leaks before exiting */
+#endif
     return 0;
 }
