@@ -20,7 +20,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include	<config.h>
 #include	<stdio.h>
 #include	<stdlib.h>
-#include	<unistd.h>
 #include	<string.h>
 #include	<fcntl.h>
 #include	<dirent.h>
@@ -28,8 +27,17 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include	<sys/types.h>
 #include	<sys/stat.h>
 #include	<sys/time.h>
+#include	<unistd.h>
 #include	<errno.h>
-#include	<openssl/sha.h>
+/* support platforms that don't yet conform to C99 */
+#if	HAVE_STDINT_H
+#include	<stdint.h>
+#elif	HAVE_INTTYPES_H
+#include	<inttypes.h>
+#else
+#error No stdint.h or inttypes.h found.
+#endif
+#include	"gnupg/sha1.h"
 #include	"cdb.h"
 #include	"cdb_make.h"
 #include	"cdb_get.h"
@@ -89,23 +97,55 @@ static unsigned long fileinfo_ruleset(options *opts, fileinfo *inf)
     return inf->ruleset;
 }
 
-inline static void do_checksum(options *opts, fileinfo *inf)
+/* There's a race condition between lstat and open in the following
+ * sequence:
+ *
+ *	lstat "foo"
+ *	if is regular file, open and read
+ *
+ * ... so we do an extra (cheap, says Andrew Gierth) stat using fstat
+ * on the file descriptor returned by open.  Then we can avoid accidentally
+ * following symlinks or reading from fifos, sockets, etc., that have
+ * become file "foo" (e.g. with "mv mysock foo") since we did lstat.
+ *
+ * Thanks to lumpy <lumpy@9mm.com> for pointing out the race condition.
+ *
+ */
+inline static int do_checksum(options *opts, fileinfo *inf)
 {
-    SHA_CTX	context;
+    SHA1_CONTEXT	context;
     char	buf[BUFSIZ];
     int		n;
-    int		fd	 = open(inf->path, O_RDONLY);
+    struct stat	fd_sb;		/* file-descriptor-based stat buffer */
+    int		fd;
     dbinfo	*dbinf	 = &inf->dbinf;
     struct utimbuf	utb;	/* for resetting access time */
 
-    if (fd == -1)
-      die(__FUNCTION__, "Error: opening file (%s): %s",
+    if ( (fd = open(inf->path, O_RDONLY)) == -1) {
+      warn(__FUNCTION__, "Warning: could not open file (%s) for reading: %s",
+	   inf->path, strerror(errno));
+      return 0;			/* didn't do checksum */
+    }
+    if (fstat(fd, &fd_sb) == -1)
+      die(__FUNCTION__, "Error: could not fstat file (%s): %s\n",
 	  inf->path, strerror(errno));
-    SHA1_Init(&context);
+
+    /* check that the file we've opened is the same one we did lstat on */
+    if (fd_sb.st_ino != inf->dbinf.stat.st_ino) {
+      warn(__FUNCTION__, "Warning: file (%s) changed before we opened it\n",
+	   inf->path);
+      return 0;			/* didn't do checksum */
+    }
+
+    sha1_init(&context);
     while ( (n = read(fd, buf, BUFSIZ)) )
-      SHA1_Update(&context, buf, n);
+      sha1_write(&context, buf, n);
     close(fd);
-    SHA1_Final(dbinf->sum, &context);
+    sha1_final(&context);
+    /* It is faster but slightly dirtier to use context.buf directly.
+     */
+    /* memcpy(dbinf->sum, sha1_read(&context), SHA1_DIGEST_LENGTH); */
+    memcpy(dbinf->sum, context.buf, SHA1_DIGEST_LENGTH);
 
     /* reset access time on request */
     if (fileinfo_ruleset(opts, inf) & RULE_RESET_ATIME) {
@@ -120,6 +160,8 @@ inline static void do_checksum(options *opts, fileinfo *inf)
 	     "Warning: resetting access time for file (%s): %s",
 	     inf->path, strerror(errno));
     }
+
+    return 1;			/* did do checksum */
 }
 
 static void show_diff_xml_long(FILE *out, char T, const char *type,
@@ -188,14 +230,14 @@ static void show_diff_perms(FILE *out, mode_t old, mode_t new)
     if (S_ISLNK(old)) /* for a symlink ... */
       fputs("sym", out); /* display "sym" instead of "777" */
     else
-      fprintf(out, "%o", old & PERM_MASK);
+      fprintf(out, "%o", (unsigned int) old & PERM_MASK);
 
     putc(':', out);
 
     if (S_ISLNK(new))
       fputs("sym", out);
     else
-      fprintf(out, "%o) ", new & PERM_MASK);
+      fprintf(out, "%o) ", (unsigned int) new & PERM_MASK);
 }
 
 static void show_diffs(options *opts,
@@ -363,11 +405,11 @@ static void report_differences(options *opts, fileinfo *currinf)
       if (knownsiz != sizeof(dbinfo)) /* maybe known has no checksum */
 	report_2regfile(opts, path);
       else if (flags & RULE_SUM) {
-	if (! currinf->did_sum) {
-	  do_checksum(opts, currinf);
-	  currinf->did_sum	 = 1;
-	}
-	if (memcmp(currinf->dbinf.sum, old.sum, sizeof(old.sum)))
+	if (! currinf->did_sum)
+	  currinf->did_sum	 = do_checksum(opts, currinf);
+
+	if (currinf->did_sum
+	    && memcmp(currinf->dbinf.sum, old.sum, sizeof(old.sum)))
 	  report_sumchange(opts, path, old.sum, sizeof(old.sum),
 			   currinf->dbinf.sum, sizeof(currinf->dbinf.sum));
       }
@@ -448,16 +490,14 @@ inline static void do_update(options *opts, fileinfo *inf)
     dbinfo		*dbinf	 = &inf->dbinf;
     size_t		datasiz;
     
-    if (S_ISREG(dbinf->stat.st_mode)) {
+    if (S_ISREG(dbinf->stat.st_mode))
       /* BUGGISH: don't test inf->did_sum, since this function's called
        *          before do_check (very minorly buggish :)
        */
-      do_checksum(opts, inf);
-      inf->did_sum	 = 1;
-      datasiz		 = sizeof(*dbinf);
-    } else {
-      datasiz		 = sizeof(dbinf->stat);
-    }
+      inf->did_sum	 = do_checksum(opts, inf);
+
+    datasiz	 = inf->did_sum ? sizeof(*dbinf) : sizeof(dbinf->stat);
+
     if (cdb_put(db, inf->path, inf->pathlen, dbinf, datasiz) == -1)
       DIE("adding record to current-state db");
 }
@@ -466,7 +506,7 @@ wft_ret_t process_file(const char *path, const struct stat *sb, void *data)
 {
     options	*opts		 = (options *) data;
     fileinfo	inf;		/* initialize before use */
-    checkset	cset;
+    checkset	*cset;
     size_t	plen		 = strlen(path);
     char	*pathcopy	 = xstrdup(path);
     char	*p		 = pathcopy;
@@ -505,7 +545,7 @@ wft_ret_t process_file(const char *path, const struct stat *sb, void *data)
     cset	 = hashtbl_lookup(opts->ruleset, p, plen);
 
     /* see whether checkset includes boolean to ignore the file */
-    if (cset && (CHECKSET_GETIGNORE(cset))) { 
+    if (cset && (cset->flags & RULE_IGNORE)) { 
 #ifdef	DEBUG
       fprintf(stderr, "debug: ignoring file (%s)\n", p);
 #endif
@@ -524,7 +564,7 @@ wft_ret_t process_file(const char *path, const struct stat *sb, void *data)
     free(pathcopy);
 
     /* see whether the nochildren boolean is true in this checkset */
-    if (cset && (CHECKSET_GETNOCHILD(cset))) {
+    if (cset && (cset->flags & RULE_NOCHILDREN)) {
 #ifdef	DEBUG
       fprintf(stderr, "debug: no children for file (%s)\n", path);
 #endif
