@@ -28,6 +28,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  * ... as the docs say they might.
  */
 
+/* This file tree walker silently ignores files that vanish between the
+ * time the readdir is done and the time the lstat is done.
+ */
 #include	<config.h>
 #include	<stdio.h>
 #include	<stdlib.h>
@@ -110,9 +113,6 @@ int walk_file_tree(const char *rootname, wft_cb_t callback, void *cbdata)
     wftnode		*np;
     char		*curr_dirname	 = xstrdup(rootname);
     char		*path		 = NULL;
-#if	!(defined(__FreeBSD__) || defined(__sun))
-    struct dirent	entry;
-#endif
     struct dirent	*result;
     struct stat		statbuf;
     int			ret, cb_ret;
@@ -127,12 +127,7 @@ int walk_file_tree(const char *rootname, wft_cb_t callback, void *cbdata)
 
       /* while we don't want to go deeper into file tree */
       while (! np->next) {
-#if	defined(__FreeBSD__) || defined(__sun)
 	result = readdir(np->dir);
-#else
-	if (readdir_r(np->dir, &entry, &result) != 0)
-	  return -1;
-#endif
 	if (!result) {
 	  /* no more entries in this dir */
 	  if (finishdir(np, &dirs, &done, curr_dirname) == -1)
@@ -143,7 +138,10 @@ int walk_file_tree(const char *rootname, wft_cb_t callback, void *cbdata)
 	  free(path);
 	path	 = xstradd(curr_dirname, "/", result->d_name, NULL);
 	if ( (ret = is_directory(path, &statbuf)) == -1) {
-	  return -1;
+	  if (errno == ENOENT)	/* no such file or directory */
+	    continue;		/* silently skip the vanished file */
+	  else
+	    return -1;
 	} else if (ret == 1) {
 	  if (! strcmp(result->d_name, ".")) { /* don't opendir on self */
 	    cb_ret	 = callback(path, &statbuf, cbdata);
