@@ -1,5 +1,5 @@
 /*
-hashtbl
+hashtbl - a handy and fast, free hash table implementation
 Copyright (C) 2001 Ed Cashin
 
 This program is free software; you can redistribute it and/or
@@ -22,11 +22,13 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 #include	<stdio.h>
 #include	<stdlib.h>
-/* until these platforms conform to C99 */
-#if defined(__FreeBSD__) || defined(__sun)
+/* support platforms that don't yet conform to C99 */
+#if	HAVE_STDINT_H
+#include	<stdint.h>
+#elif	HAVE_INTTYPES_H
 #include	<inttypes.h>
 #else
-#include	<stdint.h>
+#error No stdint.h or inttypes.h found.
 #endif
 #include	<errno.h>
 #include	"hashtbl.h"
