@@ -17,7 +17,7 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 */
-/* $Header: /cvsroot/integrit/integrit/main.c,v 1.12 2000/12/31 22:17:29 ecashin Exp $ */
+/* $Header: /cvsroot/integrit/integrit/main.c,v 1.17 2001/06/17 21:36:42 ecashin Exp $ */
 #include	<config.h>
 #include	<stdio.h>
 #include	<unistd.h>
@@ -26,7 +26,15 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include	<errno.h>
 #include	<fcntl.h>
 #include	<sys/stat.h>
-#include	<openssl/md5.h>
+/* support platforms that don't yet conform to C99 */
+#if	HAVE_STDINT_H
+#include	<stdint.h>
+#elif	HAVE_INTTYPES_H
+#include	<inttypes.h>
+#else
+#error No stdint.h or inttypes.h found.
+#endif
+#include	"gnupg/md5.h"
 #include	"cdb.h"
 #include	"cdb_make.h"
 #include	"hashtbl/hashtbl.h"
@@ -63,14 +71,15 @@ static void open_known_cdb(options *opts)
 
 static void open_current_cdb(options *opts)
 {
-    int			fd;
+    FILE	*newdb	 = NULL; /* get rid of superfluous compiler warning */
+    int		fd;
 
     fd	 = open(opts->currdbname,
 		O_TRUNC | O_CREAT | O_WRONLY | O_NDELAY, 0640);
-    if (fd == -1)
+    if ((fd == -1) || ! (newdb = fdopen(fd, "wb")) )
       DIE("opening current-state database");
     memset(&opts->currdb, 0, sizeof(opts->currdb));
-    if (cdb_make_start(&opts->currdb, fd) == -1)
+    if (cdb_make_start(&opts->currdb, newdb) == -1)
       DIE("start cdb_make");
 }
 
@@ -78,7 +87,7 @@ static void close_current_cdb(options *opts)
 {
     if (cdb_make_finish(&opts->currdb) == -1)
       DIE("finishing current-state database");
-    (void) close(opts->currdb.fd);
+    (void) fclose(opts->currdb.fout);
 }    
 
 static void close_known_cdb(options *opts)
@@ -89,7 +98,7 @@ static void close_known_cdb(options *opts)
 
 static void get_currdb_checksum(options *opts, char sumbuf[])
 {
-    MD5_CTX	context;
+    MD5_CONTEXT	context;
     char	buf[BUFSIZ];
     char	*fname	 = opts->currdbname;
     long	n;
@@ -98,15 +107,21 @@ static void get_currdb_checksum(options *opts, char sumbuf[])
     if (fd == -1)
       die(__FUNCTION__, "Error: opening file (%s): %s",
 	  fname, strerror(errno));
-    MD5_Init(&context);
+    md5_init(&context);
     while ( (n = read(fd, buf, BUFSIZ)) ) {
       if (n == -1)
 	die(__FUNCTION__, "Error: reading file (%s): %s",
 	    fname, strerror(errno));
-      MD5_Update(&context, buf, (unsigned long) n);
+      md5_write(&context, buf, (unsigned long) n);
     }
     (void) close(fd);
-    MD5_Final(sumbuf, &context);
+    md5_final(&context);
+
+    /* It would be faster to use context.buf directly, but we only
+     * do an md5 once.
+     */
+    memcpy(sumbuf, md5_read(&context), MD5_DIGEST_LENGTH);
+    /* memcpy(sumbuf, context.buf, MD5_DIGEST_LENGTH); */
 }
 
 static void show_currdb_checksum_abbrev(options *opts, char *sumbuf, size_t n)
@@ -172,7 +187,7 @@ int main(int argc, char *argv[])
 	check_for_missing(&opts);	/* only do this after new current cdb
 					   is closed (i.e. cdb_make is done) */
       else if (opts.verbose > 0)
-	fputs(PROGNAME ": not doing update, so no check for missing files",
+	fputs(PROGNAME ": not doing update, so no check for missing files\n",
 	      stderr);
       
       close_known_cdb(&opts);
